/*
 * Copyright (c) 2003 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <libxfce4mcs/mcs-manager.h>
#include <libxfce4util/debug.h>
#include <libxfce4util/i18n.h>
#include <libxfce4util/util.h>
#include <libxfcegui4/libxfcegui4.h>
#include <xfce-mcs-manager/manager-plugin.h>

#include <xfce4-session/shutdown.h>

#include "session-icon.h"

/* */
#define	RCDIR		"settings"
#define CHANNEL		"session"
#define	RCFILE		"session.xml"

/* prototypes */
static void	run_dialog(McsPlugin *);
static gboolean	save_settings(McsPlugin *);
static void	confirmLogoutChangedCB(GtkWidget *, McsPlugin *);
static void	autoSaveChangedCB(GtkWidget *, McsPlugin *);

/* settings */
static gboolean	confirmLogout = TRUE;
static gboolean	autoSave = FALSE;

/* */
static GtkWidget *dialog = NULL;

/*
 */
McsPluginInitResult
mcs_plugin_init(McsPlugin *plugin)
{
	McsSetting *setting;
	gchar *file;

	xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

	file = xfce_get_userfile(RCDIR, RCFILE, NULL);
	mcs_manager_add_channel_from_file(plugin->manager, CHANNEL, file);
	g_free(file);

	if ((setting = mcs_manager_setting_lookup(plugin->manager,
					"Session/ConfirmLogout",
					CHANNEL)) != NULL) {
		confirmLogout = setting->data.v_int;
	}
	else {
		mcs_manager_set_int(plugin->manager, "Session/ConfirmLogout",
				CHANNEL, confirmLogout);
	}

	if ((setting = mcs_manager_setting_lookup(plugin->manager,
					"Session/AutoSave",
					CHANNEL)) != NULL) {
		autoSave = setting->data.v_int;
	}
	else {
		mcs_manager_set_int(plugin->manager, "Session/AutoSave",
				CHANNEL, autoSave);
	}

	plugin->plugin_name = g_strdup("session");
	plugin->caption = g_strdup(_("Session management"));
	plugin->run_dialog = run_dialog;
	plugin->icon = inline_icon_at_size(session_icon_data, 48, 48);

	return(MCS_PLUGIN_INIT_OK);
}

/*
 */
static gboolean
save_settings(McsPlugin *plugin)
{
	gboolean result;
	gchar *file;

	file = xfce_get_userfile(RCDIR, RCFILE, NULL);
	result = mcs_manager_save_channel_to_file(plugin->manager, CHANNEL,
			file);
	g_free(file);

	return(result);
}

/*
 */
static void
confirmLogoutChangedCB(GtkWidget *widget, McsPlugin *plugin)
{
	confirmLogout = gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(widget));

	mcs_manager_set_int(plugin->manager, "Session/ConfirmLogout",
			CHANNEL, confirmLogout);
	mcs_manager_notify(plugin->manager, CHANNEL);
	save_settings(plugin);
}

/*
 */
static void
autoSaveChangedCB(GtkWidget *widget, McsPlugin *plugin)
{
	autoSave = gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(widget));

	mcs_manager_set_int(plugin->manager, "Session/AutoSave",
			CHANNEL, autoSave);
	mcs_manager_notify(plugin->manager, CHANNEL);
	save_settings(plugin);
}

/*
 */
static void
responseCB(McsPlugin *plugin)
{
	save_settings(plugin);
	gtk_widget_destroy(dialog);
	dialog = NULL;
}

/*
 */
static void
run_dialog(McsPlugin *plugin)
{
	GtkWidget *checkbox;
	GtkWidget *header;
	GtkWidget *align;
	GtkWidget *frame;
	GtkWidget *vbox;

	if (dialog != NULL) {
		gtk_window_present(GTK_WINDOW(dialog));
		return;
	}

	dialog = gtk_dialog_new_with_buttons(_("Session management"), NULL,
			GTK_DIALOG_NO_SEPARATOR,
			GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
			NULL);

	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);

	g_signal_connect_swapped(dialog, "response", G_CALLBACK(responseCB), 
			plugin);
	g_signal_connect_swapped(dialog, "delete-event",G_CALLBACK(responseCB),
			plugin);

	header = create_header(plugin->icon, _("Session management"));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), header, FALSE,
			TRUE, 0);
	gtk_widget_show(header);

	/* */
	align = gtk_alignment_new(0, 0, 0, 0);
	gtk_widget_set_size_request(align, 6, 6);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), align, FALSE,
			TRUE, 0);
	gtk_widget_show(align);

	/* Startup settings */
	frame = xfce_framebox_new(_("Logout"), TRUE);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, TRUE,
			TRUE, 0);
	gtk_widget_show(frame);

	vbox = gtk_vbox_new(FALSE, 6);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
	gtk_widget_show(vbox);

	/* */
	checkbox = gtk_check_button_new_with_label(_("Confirm logout"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox),
			confirmLogout);
	g_signal_connect(checkbox, "toggled",G_CALLBACK(confirmLogoutChangedCB),
			plugin);
	gtk_box_pack_start(GTK_BOX(vbox), checkbox, FALSE, TRUE, 0);
	gtk_widget_show(checkbox);

	/* */
	checkbox = gtk_check_button_new_with_label(
			_("Automatically save session on logout"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox),
			autoSave);
	g_signal_connect(checkbox, "toggled", G_CALLBACK(autoSaveChangedCB),
			plugin);
	gtk_box_pack_start(GTK_BOX(vbox), checkbox, FALSE, TRUE, 0);
	gtk_widget_show(checkbox);

	gtk_widget_show(dialog);
}

/* */
MCS_PLUGIN_CHECK_INIT
