/*
 * Copyright (c) 2003 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICEutil.h>
#include <X11/SM/SMlib.h>

#include <libxfce4util/i18n.h>
#include <libxfce4util/util.h>
#include <libxfcegui4/libxfcegui4.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>

#include "client.h"
#include "ice-layer.h"
#include "manager.h"
#include "pending.h"
#include "splash-icon.h"
#include "util.h"

/* list of restored, but not yet reconnected SM clients */
GList	*pendingClients = NULL;

/* */
static guint	pendingTimeoutId = 0;

/* */
static GtkWidget	*splashWindow = NULL;
static GtkWidget	*splashProgress;

/* */
static double		clientsToStart;
static double		clientsStarted;

/* */
#define PENDING_TIMEOUT		(5 * 1000)

/*
 */
static gboolean
pending_timeout(Client *client)
{
	/* client startup timed out */
	if (client != NULL)
		g_warning("client %s timed out", client->id);
	pending_continue(client);
	return(FALSE);
}

/*
 */
static gboolean
pending_launch(void)
{
	Client *client;
	GError *error;
	gchar **argv;
	gchar *cwd;

	argv = NULL;
	client = NULL;

	for (;;) {
		/* get the next client to fire up */
		while (g_list_length(pendingClients) != 0) {
			client = CLIENT(g_list_first(pendingClients)->data);

			if ((argv = client_get_restart_command(client)) != NULL)
				break;
			g_warning("Client %s specified no RestartCommand",
					client->id);
			pendingClients = g_list_remove(pendingClients, client);
			client_free(client);
		}

		/* check if we are finished */
		if (g_list_length(pendingClients) == 0) {
			gtk_widget_destroy(splashWindow);
			break;
		}
		
#ifdef DEBUG
		g_print("Restarting client %s\n", client->id);
#endif
		
		cwd = client_get_current_directory(client);
		
		if (!g_spawn_async(cwd, argv, NULL, G_SPAWN_SEARCH_PATH, NULL,
					NULL, NULL, &error)) {
			g_warning("Unable to start client %s: %s", client->id,
					error->message);
			g_error_free(error);
			pendingClients = g_list_remove(pendingClients, client);
			client_free(client);
			continue;
			
		}
		
		if (cwd != NULL)
			g_free(cwd);
		g_free(argv);

		/* add startup timeout */
		pendingTimeoutId = g_timeout_add(PENDING_TIMEOUT,
				(GSourceFunc)pending_timeout, client);
		break;
	}

	return(FALSE);
}

/*
 */
void
pending_continue(Client *client)
{
	if (pendingTimeoutId != 0)
		g_source_remove(pendingTimeoutId);

	if (client != NULL) {
		pendingClients = g_list_remove(pendingClients, client);
		client_free(client);
	}
	else if (splashWindow == NULL) {
		GdkPixbuf *pixbuf;
		GtkWidget *vbox;
		GtkWidget *image;

		/* Ok, start up the splash screen */
		splashWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_position(GTK_WINDOW(splashWindow),
				GTK_WIN_POS_CENTER_ALWAYS);
		gtk_window_set_decorated(GTK_WINDOW(splashWindow), FALSE);
#if 0
		gtk_window_set_type_hint(GTK_WINDOW(splashWindow),
				GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
#endif
		gtk_window_stick(GTK_WINDOW(splashWindow));
		vbox = gtk_vbox_new(FALSE, 1);
		gtk_widget_show(vbox);
		gtk_container_add(GTK_CONTAINER(splashWindow), vbox);
		pixbuf = inline_icon_at_size(splash_icon_data, 350, 350);
		image = gtk_image_new_from_pixbuf(pixbuf);
		g_object_unref(pixbuf);
		gtk_widget_show(image);
		gtk_box_pack_start(GTK_BOX(vbox), image, TRUE, TRUE, 0);
		splashProgress = gtk_progress_bar_new();
		gtk_widget_show(splashProgress);
		gtk_box_pack_start(GTK_BOX(vbox), splashProgress, FALSE,
				TRUE, 0);
		gtk_widget_show(splashWindow);

		clientsToStart = g_list_length(pendingClients);
		clientsStarted = 0.0;

		/* XXX - give time to the splash screen to appear */
		g_idle_add((GSourceFunc)pending_timeout, NULL);
		return;
	}

	/* just another client started */
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(splashProgress),
			++clientsStarted / clientsToStart);

	g_idle_add((GSourceFunc)pending_launch, NULL);
}


